package cn.jisol.ngame.frame.controller;


import cn.jisol.ngame.actions.SystemAction;
import cn.jisol.ngame.actions.TileSyncAction;
import cn.jisol.ngame.actions.entity.TileServerInfo;
import cn.jisol.ngame.proto.JNSyncMessage;
import cn.jisol.ngame.util.NewsContext;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Objects;

@Api(value = "JNGameDemo - API", tags = {"WorldCard - API"})
@RestController
@RequestMapping()
@ResponseBody
public class DemoController {


    //Tile 端口 分配
    private Integer nTilePort = 9000;

    //Id 分配
    private Integer nClientId = 1;

    //随机Id 分配
    private Integer nRandomId = 1;


    @ApiOperation(value = "获取玩家Id")
    @GetMapping("/sync/tile/client/id")
    public NewsContext<Integer> getTileClientId(){
        return NewsContext.onSuccess("查询成功",nClientId++);
    }

    @ApiOperation(value = "获取随机Id")
    @GetMapping("/sync/tile/random/id")
    public NewsContext<Integer> getTileRandomId(){
        return NewsContext.onSuccess("查询成功",nRandomId++);
    }

    @ApiOperation(value = "获取Tile端口")
    @GetMapping("/sync/tile/port")
    public NewsContext<Integer> getTilePort(){
        return NewsContext.onSuccess("查询成功",nTilePort++);
    }

    @ApiOperation(value = "获取TileId")
    @GetMapping("/sync/tile/id")
    public NewsContext<Integer> getTileId(){
        return NewsContext.onSuccess("查询成功", TileSyncAction.getNeedTileId());
    }


    @ApiImplicitParams({
            @ApiImplicitParam(name="index",value="所需Tile区域Id"),
    })
    @ApiOperation(value = "获取Tile服务器连接")
    @GetMapping("/sync/tile/server")
    public NewsContext<TileServerInfo> getTileServer(Integer index){
        return NewsContext.onSuccess("查询成功",TileSyncAction.getTileInfoMaster(index));
    }
    @ApiImplicitParams({
            @ApiImplicitParam(name="index",value="所需Tile区域Id"),
    })
    @ApiOperation(value = "获取所有Tile服务器连接组")
    @GetMapping("/sync/tile/servers")
    public NewsContext<List<TileServerInfo>> getTilesServer(Integer index){
        return NewsContext.onSuccess("查询成功",TileSyncAction.getTileInfos(index));
    }



    @ApiImplicitParams({
            @ApiImplicitParam(name="start",value="帧开始"),
            @ApiImplicitParam(name="end",value="帧结束")
    })
    @ApiOperation(value = "获取帧同步数据")
    @GetMapping("/sync/frame")
    public ResponseEntity<byte[]> getSyncFrame(Integer start, Integer end){

        HttpHeaders headers = new HttpHeaders();
        headers.set("Content-Type", "application/json");

        if(Objects.isNull(SystemAction.frame))
            return ResponseEntity.ok().headers(headers).body(new byte[0]);

        JNSyncMessage.JNFrameInfos infos = SystemAction.frame.vGetFrame(start, end);
        return ResponseEntity.ok().headers(headers).body(infos.toByteArray());
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name="start",value="帧开始"),
            @ApiImplicitParam(name="end",value="帧结束")
    })
    @ApiOperation(value = "获取帧同步数据")
    @GetMapping("/sync/hello")
    public String getSyncHello(Integer start, Integer end){
        return "HelloWorld";
    }

}
