package cn.jisol.game.controller.game;

import cfg.TB.TbGRoleEquip;
import cfg.TB.TbGRoleEquipQuality;
import cfg.TB.TbGRoleEquipType;
import cfg.TbGEntity.TAttributeValue;
import cn.hutool.core.util.RandomUtil;
import cn.jisol.game.controller.argsresolver.CurrentPlayer;
import cn.jisol.game.data.ResourceId;
import cn.jisol.game.data.TD;
import cn.jisol.game.entity.data.GAttribute;
import cn.jisol.game.entity.table.*;
import cn.jisol.game.service.EquipForgingBenchService;
import cn.jisol.game.service.PetEquipService;
import cn.jisol.game.service.PlayerPetService;
import cn.jisol.game.service.ResourceService;
import cn.jisol.game.util.GRandomUtil;
import cn.jisol.game.vo.news.NewsResource;
import cn.jisol.game.vo.news.ov.ResourceUpdateOV;
import cn.jisol.game.vo.pet.equip.EquipForgingBenchOV;
import cn.jisol.game.vo.pet.equip.PetEquipForgingOV;
import cn.jisol.ngame.util.NewsContext;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.retry.RetryException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 玩家宠物装备
 */
@Api(value = "JNGameDemo - API", tags = {"玩家宠物装备 - API"})
@RestController
@RequestMapping("/game/equip")
@ResponseBody
public class GPlayerEquipController {

    @Autowired
    PlayerPetService playerPetService;
    @Autowired
    PetEquipService petEquipService;
    @Autowired
    EquipForgingBenchService equipForgingBenchService;
    @Autowired
    ResourceService resourceService;

    //锻造装备(扣除玩家锻造石 打造一件装备)
    @ApiImplicitParams({})
    @ApiOperation(value = "锻造装备(扣除玩家锻造石 打造一件装备)")
    @GetMapping("/forging")
    @Transactional
    public NewsContext<PetEquipForgingOV> forging(@CurrentPlayer Player player){

        //扣除一个锻造石
        resourceService.addResourceValue(player.getPlayerId(), ResourceId.ForgedStones.id, -1L);

        //获取锻造台
        EquipForgingBenchOV forgingInfo = this.forgingInfo(player).data;

        if(Objects.isNull(forgingInfo.getForgingPetId())) return NewsContext.onFail("请设置锻造宠");

        //随机取一个部位
        TbGRoleEquipType position = GRandomUtil.randomArray(TD.DATA.getTbGRoleEquipType().getDataList());

        //获取品质
        TbGRoleEquipQuality qualityInfo = TD.DATA.getTbGRoleEquipQuality().get(forgingInfo.getForgingQuality());
        //根据品质概率随机品质
        int quality = GRandomUtil.odds(qualityInfo.odds) + 1;

        //筛选装备
        List<TbGRoleEquip> equips = TD.DATA.getTbGRoleEquip().getDataList().stream().filter(info -> (info.quality == quality && info.position == position.id)).collect(Collectors.toList());

        //随机取一个装备
        TbGRoleEquip equipInfo = GRandomUtil.randomArray(equips);

        //封装装备
        PetEquip equip = new PetEquip();
        equip.setEquipCfgId(equipInfo.id);
        equip.setEquipPlayerId(player.getPlayerId());
        equip.setEquipLevel(forgingInfo.getForgingLevel());
        equip.setEquipPosition(equipInfo.position);
        equip.setEquipBaseAttributes(
                Arrays.stream(equipInfo.baseAttributes).map(ranges ->
                        GAttribute.builder()
                            .id(ranges[0].id)
                            .value(RandomUtil.randomInt(ranges[0].value, ranges[1].value))
                        .build())
                    .collect(Collectors.toList())
        );
        if (equipInfo.highAttributeSize[0] != equipInfo.highAttributeSize[1]){
            equip.setEquipHighAttributes(
                    Arrays.stream(new Object[RandomUtil.randomInt(equipInfo.highAttributeSize[0],equipInfo.highAttributeSize[1] + 1)]).map(item -> {
                        //随机取一个高级属性
                        TAttributeValue[] ranges = GRandomUtil.randomArray(Arrays.asList(equipInfo.highAttributes));
                        return GAttribute.builder()
                                .id(ranges[0].id)
                                .value(RandomUtil.randomInt(ranges[0].value, ranges[1].value + 1))
                                .build();
                    }).collect(Collectors.toList())
            );
        }else{
            equip.setEquipHighAttributes(new ArrayList<>());
        }

        //宠物锻造经验+1
        forgingInfo.addForgingPetLevel(1);

        petEquipService.save(equip);
        equipForgingBenchService.saveOrUpdate(forgingInfo);

        return NewsContext.onSuccess("锻造成功", PetEquipForgingOV.builder().equip(equip).info(forgingInfo).build());

    }

    //获取所有装备
    @ApiImplicitParams({})
    @ApiOperation(value = "获取所有装备")
    @GetMapping("/all")
    public NewsContext<List<PetEquip>> all(@CurrentPlayer Player player){
        return NewsContext.onSuccess("获取成功",
                petEquipService.list(Wrappers.lambdaQuery(PetEquip.class)
                        .eq(PetEquip::getEquipPlayerId,player.getPlayerId()))
        );
    }


    //获取锻造台等级
    @ApiImplicitParams({})
    @ApiOperation(value = "获取锻造台")
    @GetMapping("/forging/info")
    public NewsContext<EquipForgingBenchOV> forgingInfo(@CurrentPlayer Player player){

        EquipForgingBench info = equipForgingBenchService.getOne(Wrappers.lambdaQuery(EquipForgingBench.class).eq(EquipForgingBench::getPlayerId,player.getPlayerId()));

        //如果没有阵法则创建一个
        if(Objects.isNull(info)){
            try{
                info = EquipForgingBench.builder().playerId(player.getPlayerId())
                        .forgingQuality(TD.DATA.getTbGRoleEquipQuality().getDataList().get(0).id)
                        .build();
                equipForgingBenchService.saveOrUpdate(info);
            }catch (Exception ignored){}
            assert equipForgingBenchService != null;
            info = equipForgingBenchService.getOne(Wrappers.lambdaQuery(EquipForgingBench.class).eq(EquipForgingBench::getPlayerId,player.getPlayerId()));
        }

        return NewsContext.onSuccess("获取成功",EquipForgingBenchOV.create(info));

    }

    //设置锻造宠
    @ApiImplicitParams({})
    @ApiOperation(value = "设置锻造宠")
    @PostMapping("/forging/petId/{petId}")
    public NewsContext<EquipForgingBenchOV> setForgingPetId(@CurrentPlayer Player player, @PathVariable Integer petId){

        EquipForgingBench info = equipForgingBenchService.getOne(Wrappers.lambdaQuery(EquipForgingBench.class).eq(EquipForgingBench::getPlayerId,player.getPlayerId()));

        if(Objects.isNull(info)){
            return NewsContext.onFail("没有找到锻造台");
        }

        info.setForgingPetId(petId);
        equipForgingBenchService.saveOrUpdate(info);

        return NewsContext.onSuccess("设置成功",EquipForgingBenchOV.create(info));

    }

    //穿戴装备
    @ApiImplicitParams({})
    @ApiOperation(value = "穿戴装备")
    @PostMapping("/wear/{petId}/{equipId}")
    public NewsContext<PetEquip> wear(@CurrentPlayer Player player, @PathVariable Integer petId, @PathVariable Integer equipId){

        //获取装备信息
        PetEquip info = petEquipService.getById(equipId);
        //获取宠物信息
        PlayerPet pet = playerPetService.getById(petId);


        if(Objects.isNull(info)) NewsContext.onFail("背包里没有这个装备");
        if(!Objects.equals(info.getEquipPlayerId(), player.getPlayerId())) NewsContext.onFail("这个装备不是你的");
        if(Objects.isNull(pet)) NewsContext.onFail("没有找到这个宠物");
        if(!Objects.equals(pet.getPetPlayerId(), player.getPlayerId())) NewsContext.onFail("这个宠物不是你的");

        //脱掉之前的部位装备
        PetEquip before = new PetEquip();
        before.setEquipPlayerId(0L);
        petEquipService.update(
                before,
                Wrappers.lambdaQuery(PetEquip.class)
                    .eq(PetEquip::getEquipPlayerId,player.getPlayerId())
                    .eq(PetEquip::getEquipPetId,petId)
                    .eq(PetEquip::getEquipPosition,info.getEquipPosition())
        );

        //穿上
        info.setEquipPetId(pet.getPetId());
        petEquipService.saveOrUpdate(info);
        //保存
        return NewsContext.onSuccess("穿戴成功",info);
    }

    //升级锻造台
    @ApiImplicitParams({})
    @ApiOperation(value = "锻造台品质升级")
    @PostMapping("/forging/up")
    public NewsResource<EquipForgingBenchOV> wear(@CurrentPlayer Player player){
        EquipForgingBenchOV info = this.forgingInfo(player).data;

        if(info.getForgingUpTimeExcess() > 0) return NewsResource.onFail("正在升级",info);

        //下一个等级
        int nextLevel = info.getForgingQuality() + 1;
        TbGRoleEquipQuality currentForgingInfo = TD.DATA.getTbGRoleEquipQuality().get(info.getForgingQuality());
        TbGRoleEquipQuality nextForgingInfo = TD.DATA.getTbGRoleEquipQuality().get(nextLevel);
        if(Objects.isNull(nextForgingInfo)) return NewsResource.onFail("已满级");

        //扣除材料
        ArrayList<ResourceUpdateOV> resources = new ArrayList<>(resourceService.deduct(player.getPlayerId(),nextForgingInfo.materials));

        info.setForgingQuality(nextLevel);
        //保存升级结束时间
        info.setForgingUpTime((new Date()).getTime() + currentForgingInfo.upTime * 1000L);

        equipForgingBenchService.saveOrUpdate(info);

        return NewsResource.onSuccess("升级成功",info,resources.toArray(new ResourceUpdateOV[0]));
    }

    //使用加速卷
    @ApiImplicitParams({})
    @ApiOperation(value = "锻造台加速")
    @PostMapping("/use/speed/{resType}")
    @Transactional
    public NewsResource<EquipForgingBenchOV> useSpeed(@CurrentPlayer Player player, @PathVariable Integer resType){
        EquipForgingBenchOV info = this.forgingInfo(player).data;

        if(Objects.isNull(resType)) return NewsResource.onFail("请选择需要使用的加速卷",info);
        if(info.getForgingUpTimeExcess() <= 0) return NewsResource.onFail("锻造台已升级成功",info);

        //扣除资源
        ArrayList<ResourceUpdateOV> resources = new ArrayList<ResourceUpdateOV>(){{add(resourceService.deductSpeedUp(player,resType));}};

        //减少时间
        info.setForgingUpTime(info.getForgingUpTime() - resourceService.timeSpeedUp(resType));

        if(!equipForgingBenchService.updateById(info)) throw new RetryException("");;

        return NewsResource.onSuccess("使用成功",info,resources.toArray(new ResourceUpdateOV[0]));

    }


}
