package cn.jisol.game.controller.game;

import cn.jisol.game.controller.argsresolver.CurrentPlayer;
import cn.jisol.game.data.ResourceId;
import cn.jisol.game.data.TD;
import cn.jisol.game.entity.table.Player;
import cn.jisol.game.entity.table.PlayerPet;
import cn.jisol.game.service.PlayerPetService;
import cn.jisol.game.service.ResourceService;
import cn.jisol.game.vo.news.NewsResource;
import cn.jisol.game.vo.news.ov.ResourceUpdateOV;
import cn.jisol.ngame.util.NewsContext;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.retry.RetryException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 玩家宠物
 */
@Api(value = "JNGameDemo - API", tags = {"PET - API"})
@RestController
@RequestMapping("/game/pet")
@ResponseBody
public class GPlayerPetController {

    @Autowired
    PlayerPetService playerPetService;
    @Autowired
    ResourceService resourceService;

    //获取玩家宠物列表
    @ApiImplicitParams({})
    @ApiOperation(value = "获取玩家宠物列表")
    @GetMapping("/list")
    public NewsContext<List<PlayerPet>> getPetList(@CurrentPlayer Player player){

        return NewsContext.onSuccess("获取成功",
                playerPetService.list(
                        Wrappers.lambdaQuery(PlayerPet.class).eq(PlayerPet::getPetPlayerId,player.getPlayerId())
                )
        );

    }

    //提升宠物星
    @ApiImplicitParams({})
    @ApiOperation(value = "提升宠物星")
    @PostMapping("/up/star/{petId}/{num}")
    @Transactional
    public NewsResource<PlayerPet> onUpStar(@CurrentPlayer Player player, @PathVariable String petId, @PathVariable Integer num){

        //判断参数
        if(Objects.isNull(num) || num <= 0 || Objects.isNull(petId)){
            return NewsResource.onFail("参数错误");
        }
        //整理刷新数据
        ArrayList<ResourceUpdateOV> resources = new ArrayList<>();

        //查询被升级的宠物
        PlayerPet pet = playerPetService.getById(petId);

        //判断宠物是否是自己的
        if(!Objects.equals(pet.getPetPlayerId(), player.getPlayerId())){
            return NewsResource.onFail("宠物不是自己的");
        }

        //扣除魂
        resources.add(ResourceUpdateOV.onUpdate(resourceService.addResourceValue(player.getPlayerId(), ResourceId.Q1S.id, (long) -num)));

        //升级星
        pet.setPetStarExp(pet.getPetStarExp() + num);

        //判断是否可以升下一星 如果可以则 升
        int upStarExp;
        while (pet.getPetStarExp() >= (upStarExp = TD.DATA.getTbGRoleUpStar().get(pet.getPetStar() + 1).merge)){
            //消耗经验
            pet.setPetStarExp(pet.getPetStarExp() - upStarExp);
            //升星
            pet.setPetStar(pet.getPetStar() + 1);
        }

        //保存宠物信息 (保存失败则重试)
        if(!playerPetService.updateById(pet)) throw new RetryException("");

        //刷新宠物
        resources.add(ResourceUpdateOV.onPet(ResourceUpdateOV.ResourceUpdateType.UPDATE,pet));

        return NewsResource.onSuccess("升星成功",null, resources.toArray(new ResourceUpdateOV[0]));

    }

}
