package cn.jisol.game.controller.game;


import cfg.TB.TbGGift;
import cfg.TbGEnum.TGiftLimit;
import cn.jisol.game.controller.argsresolver.CurrentPlayer;
import cn.jisol.game.controller.exception.ResourceNotException;
import cn.jisol.game.data.ResourceId;
import cn.jisol.game.data.TD;
import cn.jisol.game.entity.data.GResource;
import cn.jisol.game.entity.data.GReward;
import cn.jisol.game.entity.table.GiftDayRecord;
import cn.jisol.game.entity.table.GiftRecord;
import cn.jisol.game.entity.table.Player;
import cn.jisol.game.entity.table.PlayerPet;
import cn.jisol.game.service.GiftDayRecordService;
import cn.jisol.game.service.GiftRecordService;
import cn.jisol.game.service.ResourceService;
import cn.jisol.game.vo.news.NewsReward;
import cn.jisol.game.vo.news.ov.ResourceUpdateOV;
import cn.jisol.ngame.util.NewsContext;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Api(value = "JNGameDemo - API", tags = {"礼包 - API"})
@RestController
@RequestMapping("/game/gift")
@ResponseBody
public class GiftController {

    @Autowired
    ResourceService resourceService;

    @Autowired
    GiftRecordService giftRecordService;

    @Autowired
    GiftDayRecordService giftDayRecordService;

    //购买指定礼包
    @ApiImplicitParams({})
    @ApiOperation(value = "购买指定礼包")
    @PostMapping("/buy/{giftId}")
    @Transactional
    public NewsReward<Boolean> buy(@CurrentPlayer Player player, @PathVariable Integer giftId){

        TbGGift gift = TD.DATA.getTbGGift().get(giftId);

        if(Objects.isNull(gift)) return NewsReward.onFail("暂无该礼包");

        //判断是否可以购买当前礼包
        if(gift.limit == TGiftLimit.DayLimit){
            //每天限制购买
            if(gift.limitValue <= giftDayRecordService.count(Wrappers.lambdaQuery(GiftDayRecord.class).eq(GiftDayRecord::getPlayerId,player.getPlayerId()).eq(GiftDayRecord::getGiftCfgId,gift.id))){
                return NewsReward.onFail("今天买太多啦 每天再买哦");
            }
        }
        if(gift.limit == TGiftLimit.Limit){
            //每天限制购买
            if(gift.limitValue <= giftRecordService.count(Wrappers.lambdaQuery(GiftRecord.class).eq(GiftRecord::getPlayerId,player.getPlayerId()).eq(GiftRecord::getGiftCfgId,gift.id))){
                return NewsReward.onFail("当前礼包买太多了 看看其他礼包哦");
            }
        }

        resourceService.deduct(player.getPlayerId(), gift.exchanges);

        //发放奖励
        List<ResourceUpdateOV> ovs = resourceService.rewards(player.getPlayerId(), gift.rewards);


        GiftRecord giftRecord = GiftRecord.builder()
                .playerId(player.getPlayerId())
                .giftCfgId(gift.id)
                .giftCfgName(gift.name)
                .giftCfgRewards(Arrays.stream(gift.rewards).map(GReward::toBuild).collect(Collectors.toList()))
//                .giftCfgRewards(Arrays.asList(gift.rewards))
                .giftCfgExchanges(Arrays.stream(gift.exchanges).map(GResource::toBuild).collect(Collectors.toList()))
                .giftBuyingTime(new Date().getTime())
                .build();

        //保存购买记录
        giftRecordService.save(giftRecord);
        giftDayRecordService.save(GiftDayRecord.builder()
                .playerId(giftRecord.getPlayerId())
                .giftId(giftRecord.getGiftId())
                .giftCfgId(giftRecord.getGiftCfgId())
                .build());


        return NewsReward.onSuccess("购买成功",true,ovs, Arrays.asList(gift.rewards));

    }

    //查询所有充值记录
    @ApiImplicitParams({})
    @ApiOperation(value = "查询所有充值记录")
    @GetMapping("/record/all")
    public NewsContext<List<GiftRecord>> getGiftRecord(@CurrentPlayer Player player){
        return NewsContext.onSuccess("查询成功",giftRecordService.list(
                Wrappers.lambdaQuery(GiftRecord.class)
                        .eq(GiftRecord::getPlayerId,player.getPlayerId())
        ));
    }

    //查询当天购买记录
    @ApiImplicitParams({})
    @ApiOperation(value = "查询当天购买记录")
    @GetMapping("/record/day")
    public NewsContext<List<GiftDayRecord>> getGiftDayRecord(@CurrentPlayer Player player){
        return NewsContext.onSuccess("查询成功",giftDayRecordService.list(
                Wrappers.lambdaQuery(GiftDayRecord.class)
                        .eq(GiftDayRecord::getPlayerId,player.getPlayerId())
        ));
    }


}
