package cn.jisol.game.controller.game.mode;

import cfg.TB.TbGSysDungeon;
import cfg.TbGEntity.TDungeon;
import cn.jisol.game.controller.argsresolver.CurrentPlayer;
import cn.jisol.game.data.TD;
import cn.jisol.game.entity.mode.ModeDungeonType;
import cn.jisol.game.entity.table.Dungeon;
import cn.jisol.game.entity.table.Player;
import cn.jisol.game.service.DungeonService;
import cn.jisol.game.service.ResourceService;
import cn.jisol.game.table.TableUtil;
import cn.jisol.game.vo.news.NewsResource;
import cn.jisol.game.vo.news.NewsReward;
import cn.jisol.game.vo.news.ov.ResourceUpdateOV;
import cn.jisol.ngame.util.NewsContext;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.retry.RetryException;
import org.springframework.retry.annotation.Retryable;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@Api(value = "JNGameDemo - API", tags = {"副本 - API"})
@RestController
@RequestMapping("/game/mode/dungeon")
@ResponseBody
public class GDungeonController {

    @Autowired
    ResourceService resourceService;
    @Autowired
    DungeonService dungeonService;

    //获取副本信息
    @ApiImplicitParams({})
    @ApiOperation(value = "获取副本信息")
    @GetMapping("/info")
    public NewsContext<Map<Integer,Dungeon>> getInfos(@CurrentPlayer Player player){

        //获取所有副本信息
        List<Dungeon> list = dungeonService.list(Wrappers.lambdaQuery(Dungeon.class).eq(Dungeon::getPlayerId, player.getPlayerId()));

        Map<Integer,Dungeon> dungeons = new HashMap<>();
        //获取全部副本
        TD.DATA.getTbGSysDungeon().getDataList().forEach(info -> {
            Dungeon dungeon = null;
            try{
                dungeon = list.stream().filter(item -> info.id == item.getDungeonCfgId()).collect(Collectors.toList()).get(0);
            }catch (Exception ignored){}

            if(Objects.isNull(dungeon)){
                //如果没有数据则生成数据
                try{
                    //获取当前副本最小的进度
                    Object table = Objects.requireNonNull(TableUtil.getDataList(Objects.requireNonNull(TableUtil.getTDObject(info.tigs)))).get(0);
                    dungeon = Dungeon.builder()
                            .playerId(player.getPlayerId())
                            .dungeonCfgId(info.id)
                            .dungeonCfgItemId(TableUtil.getTableId(table))
                            .build();
                    dungeonService.save(dungeon);
                }catch (Exception ignored){
                    assert dungeonService != null;
                    dungeon = dungeonService.getOne(Wrappers.lambdaQuery(Dungeon.class).eq(Dungeon::getPlayerId,player.getPlayerId()).eq(Dungeon::getDungeonCfgId,info.id));
                }
            }
            dungeons.put(dungeon.getDungeonCfgId(),dungeon);
        });

        return NewsContext.onSuccess("获取成功",dungeons);
    }

    //消耗钥匙挑战副本
    @ApiImplicitParams({})
    @ApiOperation(value = "消耗钥匙挑战普通副本")
    @PostMapping("/challenge/dungeon/{dungeonId}/{dungeonItemId}")
    @Transactional
    public NewsReward<Map<Integer,Dungeon>> challengeDungeon(@CurrentPlayer Player player, @PathVariable Integer dungeonId, @PathVariable Integer dungeonItemId){

        TbGSysDungeon tbGSysDungeon = TD.DATA.getTbGSysDungeon().get(dungeonId);
        if(Objects.isNull(tbGSysDungeon)) return NewsReward.onFail("不存在该副本");
        if(!tbGSysDungeon.type.equals(ModeDungeonType.GDungeonMode)) return NewsReward.onFail("错误副本");

        //获取副本信息
        Map<Integer, Dungeon> infos = this.getInfos(player).data;

        //禁止越级挑战
        if(infos.get(dungeonId).getDungeonCfgItemId() < dungeonItemId) return NewsReward.onFail("请不要越级挑战哦");

        //获取副本
        TableUtil.TableData tableData = Objects.requireNonNull(TableUtil.getTableData(tbGSysDungeon.tigs));
        //获取等级数据
        TDungeon dungeon = Objects.requireNonNull((TDungeon) tableData.get(dungeonItemId).getData());


        //扣除消耗
        List<ResourceUpdateOV> ovs = new ArrayList<ResourceUpdateOV>(){
            {
                add(ResourceUpdateOV.onUpdate(resourceService.addResourceValue(player.getPlayerId(), tbGSysDungeon.consume.id, (long) -tbGSysDungeon.consume.value)));
            }
        };

        ovs.addAll(resourceService.rewards(player.getPlayerId(), dungeon.rewards));

        //更新等级
        if(infos.get(dungeonId).getDungeonCfgItemId().equals(dungeonItemId)){
            if(Objects.nonNull(tableData.get(dungeonItemId + 1))){
                infos.get(dungeonId).setDungeonCfgItemId(dungeonItemId + 1);
                dungeonService.saveOrUpdate(infos.get(dungeonId));
            }
        }

        //发放奖励
        return NewsReward.onSuccess("挑战成功",infos,ovs,Arrays.asList(dungeon.rewards));

    }

}
