package cn.jisol.game.entity.mode;

import cfg.TbGEntity.TOnHookLevel;
import cn.hutool.json.JSONUtil;
import cn.jisol.game.data.TD;
import cn.jisol.game.table.GTbGOnHookLevels;
import cn.jisol.game.table.TableUtil;
import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 玩家 无限模式(OnHook) 信息
 */
@Builder
@Data
@TableName("`mode_on_hook`")
@NoArgsConstructor
@AllArgsConstructor
public class ModeOnHook {

    @TableId(type = IdType.INPUT)
    private Long playerId;          //玩家Id
    private Integer onHookMap;      //当前玩家所在的地图(配置表Id)
    private String onHookInfo;  //地图信息Json

    @TableField(exist = false)
    private ModeOnHookInfo hookInfo;  //地图信息类
    @TableField(exist = false)
    private Integer levelId;  //当前地图Id

    public String getOnHookInfo() {
        if (Objects.nonNull(hookInfo)) this.onHookInfo = JSONUtil.toJsonStr(this.hookInfo);
        if (Objects.isNull(this.onHookInfo)) this.onHookInfo = JSONUtil.toJsonStr(new ModeOnHookInfo());
        return this.onHookInfo;
    }

    public ModeOnHookInfo getHookInfo(){
        if(Objects.isNull(hookInfo)) this.hookInfo = JSONUtil.toBean(this.getOnHookInfo(), ModeOnHookInfo.class);
        return this.hookInfo;
    }

    //获取当前地图类
    @JsonIgnore
    public GTbGOnHookLevels getGTbGOnHookLevels(){
        String sign = TD.DATA.getTbGOnHookMaps().get(this.getOnHookMap()).sign;
        return new GTbGOnHookLevels(TableUtil.getTDObject(sign));
    }

    //获取当前关卡Id
    public Integer getLevelId(){
        //获取玩家当前地图关卡Id
        Integer levelId = this.getHookInfo().getMaps().get(this.getOnHookMap());
        //如果没有则默认当前地图的第一个Id
        if(Objects.isNull(levelId)) {
            List<GTbGOnHookLevels.GTbGOnHookLevel> dataList = this.getGTbGOnHookLevels().getDataList();
            if(dataList.size() == 0) return -1;
            levelId = dataList.get(0).getId();
        }
        return levelId;
    }

    //是否战胜当前关卡
    public boolean isWinLevel(){
        if(Objects.isNull(getHookInfo().getMapStates().get(getOnHookMap()))) return false;
        return getHookInfo().getMapStates().get(getOnHookMap()) >= 4;
    }

}
