package cn.jisol.game.table;

import cn.jisol.game.data.TD;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

//文档工具类
public class TableUtil {

    public static class TableData{
        //表数据
        Object data;

        public TableData(Object data) {
            this.data = data;
        }

        public Table get(Integer Id){
            return new Table(TableUtil.getId(this.data,Id));
        }

        public List<Table> getDataList(){
            return Objects.requireNonNull(TableUtil.getDataList(this.data)).stream().map(Table::new).collect(Collectors.toList());
        }

    }

    public static class Table{

        //表行数据
        Object data;

        public Table(Object data) {
            this.data = data;
        }

        public Integer getId(){
            return TableUtil.getTableId(this.data);
        }

        public Object getData(){
            return TableUtil.getTableKey(this.data,"data");
        }

    }

    //获取对象
    public static TableData getTableData(String sign){
        return new TableData(TableUtil.getTDObject(sign));
    }

    //获取名字的对象
    public static Object getTDObject(String sign){
        try {
            Method method = TD.DATA.getClass().getMethod("get" + sign);
            return method.invoke(TD.DATA);
        } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
            return null;
        }
    }

    //获取配置列表
    public static List<Object> getDataList(Object config){
        try {
            Method method = null;
            method = config.getClass().getMethod("getDataList");
            Object data = method.invoke(config);
            if(Objects.isNull(data) || !(data instanceof ArrayList)){
                return null;
            }
            return (ArrayList<Object>)data;
        } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
            return null;
        }
    }

    //通过Id获取表数据
    public static Object getId(Object config,Integer id){

        try {
            Method method = config.getClass().getMethod("get",int.class);
            Object data = method.invoke(config,id);
            if(Objects.isNull(data)) return null;
            return data;
        } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
            return null;
        }

    }

    //获取数据Id
    public static int getTableId(Object table){

        Object id = TableUtil.getTableKey(table,"id");
        if (Objects.isNull(id)){
            return -1;
        }
        return (int) id;

    }

    //获取指定Key数据
    public static Object getTableKey(Object table,String key){

        try {
            Field idField = table.getClass().getField(key);
            return idField.get(table);
        } catch (NoSuchFieldException | IllegalAccessException e) {
            e.printStackTrace();
            return null;
        }

    }

}
